/*******************************************************************************

  Defines Filter Interface Utils for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    06/23/2004 23:37 - First Creation

*******************************************************************************/

#ifndef DIV_FILTER_UTILS_H
#define DIV_FILTER_UTILS_H

#include "DIVFilterDefs.h"

#ifndef uint
#define uint unsigned int
#endif

#ifndef uchar
#define uchar unsigned char
#endif

DIM_UCHAR iTrimUC (int num);



//------------------------------------------------------------------------------
// @@@ SELECTIONS 
//------------------------------------------------------------------------------
// w - image width, y - line for which to get constrains
// pBeg and pEnd are retrieved begining and ending points
// pI - initial position in the array, start as 0 and then automatically advanced
//void findLineBegEnd(TDIVParameters *par, uint w, long y, uint &pI, uint& pBeg, uint& pEnd);
void findLineBegEndTopBot(TDIVParameters *par, uint w, long y, uint &pI, uint& pBeg, uint& pEnd);

void findLineBegEndTopBotI(TDIVParameters *par, uint w, long y, uint &pI, int& pBeg, int& pEnd);

//bool isPixelSelected( TDIVParameters *par, uint w, uint &pI, long y, long x );

//------------------------------------------------------------------------------
// @@@ SAFE CALLBACKS CALLS
//------------------------------------------------------------------------------
void showProgress( TDIVParameters *par, long done, long total, char *descr );
int testAbort( TDIVParameters *par );
void showError( TDIVParameters *par, int val, char *descr );

//------------------------------------------------------------------------------
// @@@ ARGUMENTS
//------------------------------------------------------------------------------
bool getUIntArg   (TDIVParameters *par, uint i, uint &arg );
bool getIntArg    (TDIVParameters *par, uint i, int &arg );
bool getByteArg   (TDIVParameters *par, uint i, uchar &arg );
bool getFloatArg  (TDIVParameters *par, uint i, float &arg );
bool getDoubleArg (TDIVParameters *par, uint i, double &arg );
bool getPointerArg(TDIVParameters *par, uint i, void **arg );
bool getBoolArg   (TDIVParameters *par, uint i, bool &arg );


//------------------------------------------------------------------------------
// @@@ ARGUMENT LIST CREATION
//------------------------------------------------------------------------------
TDIVArguments createFilterArguments(int count );
void freeFilterArguments( TDIVArguments *args );
void changeFilterArgument( TDIVArguments *args, uint index,
                           DWORD tagId, DWORD tagType, DWORD tagCount,
                           char *tagDescr, void *tagData );


//------------------------------------------------------------------------------
// PALETTE
//------------------------------------------------------------------------------

HPALETTE createGreyPalette ( );
HPALETTE createHPalette ( int num_colors, RGBQUAD *bmiColors );


//------------------------------------------------------------------------------------------------
// image
//------------------------------------------------------------------------------------------------
DIM_UINT   getHBLineSizeInBytes( DIM_UINT w, DIM_UINT depth );
DIM_UCHAR *getHBScanLine( DIM_UINT y, DIM_UCHAR *p, DIM_UINT w, DIM_UINT depth );

// depth is in bits per pixel, return a pointer to image data area
HBITMAP dimCreateHBitmap( DIM_UINT w, DIM_UINT h, DIM_UINT depth, int num_colors, RGBQUAD *lut, DIM_UCHAR **p, HPALETTE &hpal );
HBITMAP dimCopyHBitmap( DIM_UCHAR *src, DIM_UINT w, DIM_UINT h, DIM_UINT depth, int num_colors, RGBQUAD *lut, DIM_UCHAR **p );


#endif //DIV_FILTER_UTILS_H
