/*******************************************************************************

  Defines Format Utils for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    06/23/2004 23:37 - First Creation

  ver: 1

*******************************************************************************/

#include "DIVFormatDefs.h"
#include "DIVFormatUtils.h"

#include <math.h>
#include <stdio.h>
#include <string.h>



BYTE trimToByte(int val)
{
  if (val>255) return 255;
  if (val<0) return 0;
  return (BYTE) val;
}


//------------------------------------------------------------------------------
// Init parameters
//------------------------------------------------------------------------------

TDivFormatHandle initTDivFormatHandle()
{
  TDivFormatHandle tp;
  tp.ver = sizeof(TDivFormatHandle);
  
  tp.showProgressProc = NULL;
  tp.showErrorProc = NULL;
  tp.testAbortProc = NULL;
  tp.mallocProc = NULL;
  tp.freeProc = NULL;  

  tp.magic = 0;
  tp.magic_length = 0;
  tp.subFormat = 0;
  tp.pageNumber = 0;
  tp.resolutionLevel = 0;
  tp.quality = 100;
  tp.compression = 0;
  tp.order = 0;
  tp.roiX = 0;
  tp.roiY = 0;
  tp.roiW = 0;
  tp.roiH = 0;
  tp.metaData.count = 0;
  tp.metaData.tags = NULL;

  tp.imageServicesProcs = NULL;
  tp.internalParams = NULL;
  tp.hwndParent = 0;
  tp.hDllInstance = 0;
  
  tp.fileName = NULL;
  tp.stream  = NULL;
  tp.io_mode = DIV_IO_READ;

  tp.readProc  = NULL;
  tp.writeProc = NULL;
  tp.flushProc = NULL;
  tp.seekProc  = NULL;
  tp.sizeProc  = NULL;
  tp.tellProc  = NULL;
  tp.eofProc   = NULL;
  tp.closeProc = NULL;

  tp.hbmp = 0;
  tp.hpal = 0;

  tp.imgInfo = NULL;

  tp.param1 = NULL;
  tp.param2 = NULL;
  
  return tp;
}
  

TDivImageInfo initTDivImageInfo()
{
  TDivImageInfo tp;
  tp.ver = sizeof(TDivImageInfo);
  
  tp.width = 0;
  tp.height = 0;
  
  tp.number_pages = 0;
  tp.number_levels = 0;
    
  tp.transparentIndex = 0;
  tp.transparencyMatting = 0;
    
  //tp.imageMode = 0;
  tp.samples = 0;
  tp.depth = 0;

  tp.resUnits = 0;  
  tp.xRes = 0;
  tp.yRes = 0;
  
  tp.tileWidth = 0;
  tp.tileHeight = 0;

  return tp;
}

//------------------------------------------------------------------------------
// Safe calls for memory/io prototypes, if they are not supplied then
// standard functions are used
//------------------------------------------------------------------------------

void* dimMalloc( TDivFormatHandle *fmtHndl, DIM_ULONG size )
{
  if ( fmtHndl->mallocProc != NULL ) 
    return fmtHndl->mallocProc ( size );
  else
  {
    void *p = (void *) new char[size];
    return p;
  }
}

void* dimFree( TDivFormatHandle *fmtHndl, void *p )
{
  if ( fmtHndl->freeProc != NULL ) 
    return fmtHndl->freeProc( p );
  else
  {
    unsigned char *pu = (unsigned char*) p;
    if (p != NULL) delete pu;  
    return NULL;
  }
}


DIM_ULONG dimRead ( TDivFormatHandle *fmtHndl, void *buffer, DIM_ULONG size, DIM_ULONG count )
{
  if ( fmtHndl->stream == NULL ) return 0;
  //if ( fmtHndl->io_mode != DIM_IO_READ ) return 0;
  if ( fmtHndl->readProc != NULL ) 
    return fmtHndl->readProc( buffer, size, count, fmtHndl->stream );
  else
    return fread( buffer, size, count, (FILE *) fmtHndl->stream );
}

DIM_ULONG dimWrite ( TDivFormatHandle *fmtHndl, void *buffer, DIM_ULONG size, DIM_ULONG count )
{
  if ( fmtHndl->stream == NULL ) return 0;    
  //if ( fmtHndl->io_mode != DIM_IO_WRITE ) return 0;
  if ( fmtHndl->writeProc != NULL ) 
    return fmtHndl->writeProc( buffer, size, count, fmtHndl->stream );
  else
    return fwrite( buffer, size, count, (FILE *) fmtHndl->stream );
}

DIM_INT dimFlush ( TDivFormatHandle *fmtHndl )
{
  if ( fmtHndl->stream == NULL ) return EOF;  
  if ( fmtHndl->flushProc != NULL ) 
    return fmtHndl->flushProc( fmtHndl->stream );
  else
    return fflush( (FILE *) fmtHndl->stream );
}

DIM_INT dimSeek ( TDivFormatHandle *fmtHndl, DIM_LONG offset, DIM_INT origin )
{
  if ( fmtHndl->stream == NULL ) return 1;
  if ( fmtHndl->seekProc != NULL ) 
    return fmtHndl->seekProc( fmtHndl->stream, offset, origin );
  else
    return fseek( (FILE *) fmtHndl->stream, offset, origin );
}

DIM_ULONG dimSize ( TDivFormatHandle *fmtHndl )
{
  if ( fmtHndl->stream == NULL ) return 1;   
  long fsize = 0; 
  if ( fmtHndl->sizeProc != NULL ) 
    return fmtHndl->sizeProc( fmtHndl->stream );
  else
    return ((fsize = fseek( (FILE *) fmtHndl->stream, 0, SEEK_END )) < 0 ? 0 : fsize);
}

DIM_LONG dimTell ( TDivFormatHandle *fmtHndl )
{
  if ( fmtHndl->stream == NULL ) return -1;  
  if ( fmtHndl->tellProc != NULL ) 
    return fmtHndl->tellProc( fmtHndl->stream );
  else
    return ftell( (FILE *) fmtHndl->stream );
}

DIM_INT dimEof ( TDivFormatHandle *fmtHndl )
{
  if ( fmtHndl->stream == NULL ) return 1;
  if ( fmtHndl->eofProc != NULL ) 
    return fmtHndl->eofProc( fmtHndl->stream );
  else
    return feof( (FILE *) fmtHndl->stream );
}

DIM_INT dimClose ( TDivFormatHandle *fmtHndl )
{
  if ( fmtHndl->stream == NULL ) return EOF;
  DIM_INT res;
  if ( fmtHndl->closeProc != NULL ) 
    res = fmtHndl->closeProc( fmtHndl->stream );
  else
    res = fclose( (FILE *) fmtHndl->stream );

  fmtHndl->stream = NULL;
  return res;
}



void showProgress( TDivFormatHandle *fmtHndl, long done, long total, char *descr )
{
  if (fmtHndl == NULL) return;
  if (fmtHndl->showProgressProc == NULL) return;

  fmtHndl->showProgressProc( done, total, descr );
}

int testAbort( TDivFormatHandle *fmtHndl )
{
  if (fmtHndl == NULL) return 0;
  if (fmtHndl->testAbortProc == NULL) return 0;

  return fmtHndl->testAbortProc();
}

void showError( TDivFormatHandle *fmtHndl, int val, char *descr )
{
  if (fmtHndl == NULL) return;
  if (fmtHndl->showErrorProc == NULL) return;

  fmtHndl->showErrorProc( val, descr );
}




//------------------------------------------------------------------------------
// SWAP TYPES
//------------------------------------------------------------------------------

void dimSwapShort(DIM_UINT16* wp)
{
  register DIM_UCHAR* cp = (DIM_UCHAR*) wp;
  DIM_UCHAR t;

  t = cp[1]; cp[1] = cp[0]; cp[0] = t;
}

void dimSwapLong(DIM_UINT32* lp)
{
  register DIM_UCHAR* cp = (DIM_UCHAR*) lp;
  DIM_UCHAR t;

  t = cp[3]; cp[3] = cp[0]; cp[0] = t;
  t = cp[2]; cp[2] = cp[1]; cp[1] = t;
}

void dimSwapArrayOfShort(DIM_UINT16* wp, register DIM_ULONG n)
{
  register DIM_UCHAR* cp;
  register DIM_UCHAR t;

  while (n-- > 0) 
  {
    cp = (DIM_UCHAR*) wp;
    t = cp[1]; cp[1] = cp[0]; cp[0] = t;
    wp++;
  }
}

void dimSwapArrayOfLong(register DIM_UINT32* lp, register DIM_ULONG n)
{
  register unsigned char *cp;
  register unsigned char t;

  while (n-- > 0) 
  {
    cp = (unsigned char *)lp;
    t = cp[3]; cp[3] = cp[0]; cp[0] = t;
    t = cp[2]; cp[2] = cp[1]; cp[1] = t;
    lp++;
  }
}

void dimSwapDouble(double *dp)
{
  register DIM_UINT32* lp = (DIM_UINT32*) dp;
  DIM_UINT32 t;

  dimSwapArrayOfLong(lp, 2);
  t = lp[0]; lp[0] = lp[1]; lp[1] = t;
}

void dimSwapArrayOfDouble(double* dp, register DIM_ULONG n)
{
  register DIM_UINT32* lp = (DIM_UINT32*) dp;
  register DIM_UINT32 t;

  dimSwapArrayOfLong(lp, n + n);
  while (n-- > 0) 
  {
    t = lp[0]; lp[0] = lp[1]; lp[1] = t;
    lp += 2;
  }
}

void dimSwapData(int type, long size, void* data)
{
  if ( (type == DIM_TAG_SHORT) || (type == DIM_TAG_SSHORT) )
    dimSwapArrayOfShort( (DIM_UINT16*) data, size );

  if ( (type == DIM_TAG_LONG) || (type == DIM_TAG_SLONG) || (type == DIM_TAG_FLOAT) )
    dimSwapArrayOfLong( (DIM_UINT32*) data, size );

  if (type == DIM_TAG_RATIONAL)
    dimSwapArrayOfLong( (DIM_UINT32*) data, size );

  if (type == DIM_TAG_DOUBLE)
    dimSwapArrayOfDouble( (double*) data, size );
}

//------------------------------------------------------------------------------
// Metadata
//------------------------------------------------------------------------------

void initMetaTag(TDivTagItem *tagItem)
{
  tagItem->tagGroup = 0;  
  tagItem->tagId = 0;     
  tagItem->tagType = 0;  
  tagItem->tagLength = 0; 
  tagItem->tagData = NULL;
}

bool isTagPresent( TDivTagList *tagList, int group, int tag )
{
  if (tagList == NULL) return false;
  if (tagList->count == 0) return false;
  if (tagList->tags == NULL) return false;
  
  DIM_UINT i;
  for (i=0; i<tagList->count; i++)
    if ( (tagList->tags[i].tagId == (DIM_UINT) tag) && 
         (tagList->tags[i].tagGroup == (DIM_UINT) group) ) return true;
  
  return false;
}

void clearMetaTag( TDivTagItem *tagItem )
{
  if (tagItem == NULL) return;
  if (tagItem->tagLength == 0) return;
  if (tagItem->tagData == NULL) return;

  if (tagItem->tagLength * divTypeSizeBytes[tagItem->tagType] > sizeof(void*) )
    delete tagItem->tagData;

  initMetaTag(tagItem);
}


void clearMetaTag( TDivTagList *tagList, int pos )
{
  if (tagList == NULL) return;
  if (tagList->count <= pos) return;
  if (tagList->tags == NULL) return;
  if (pos < 0) return;
  clearMetaTag( &tagList->tags[pos] );
}

void clearMetaTag( TDivTagList *tagList, int group, int tag )
{
  int pos = tagPos(tagList, group, tag);
  clearMetaTag( tagList, pos );
}

void clearMetaTag( TDivFormatHandle *fmtHndl, TDivTagItem *tagItem )
{
  if (tagItem == NULL) return;
  if (tagItem->tagLength == 0) return;
  if (tagItem->tagData == NULL) return;

  if (tagItem->tagLength * divTypeSizeBytes[tagItem->tagType] > sizeof(void*) )
    dimFree( fmtHndl, tagItem->tagData );

  initMetaTag(tagItem);
}

void clearMetaTags( TDivFormatHandle *fmtHndl, TDivTagList *tagList )
{
  if (tagList == NULL) return;
  if (tagList->count == 0) return;
  if (tagList->tags == NULL) return;
  
  tagList->tags = (TDivTagItem *) dimFree( fmtHndl, tagList->tags );
  tagList->count = 0;
}

void clearMetaTagsFreeTags( TDivFormatHandle *fmtHndl, TDivTagList *tagList )
{
  if (tagList == NULL) return;
  if (tagList->count == 0) return;
  if (tagList->tags == NULL) return;
  
  int i;
  for (i=0; i<tagList->count; i++)
    clearMetaTag( fmtHndl, &tagList->tags[i] );

  tagList->tags = (TDivTagItem *) dimFree( fmtHndl, tagList->tags );
  tagList->count = 0;
}

int tagPos(TDivTagList *tagList, int group, int tag)
{
  if (tagList == NULL) return false;
  if (tagList->count == 0) return false;
  if (tagList->tags == NULL) return false;
  
  DIM_UINT i;
  for (i=0; i<tagList->count; i++)
    if  ( (tagList->tags[i].tagId == (DIM_UINT) tag) &&
          (tagList->tags[i].tagGroup == (DIM_UINT) group) )
    return i;
  
  return -1;
}

int addMetaTag( TDivFormatHandle *fmtHndl, TDivTagList *tagList, TDivTagItem item )
{
  DIM_UINT i;
  if (tagList == NULL) return 1;

  if (tagList->tags == NULL) tagList->count = 0;

  if (tagList->count != 0)
  for (i=0; i<tagList->count; i++)
    if ( (tagList->tags[i].tagId == item.tagId) &&
         (tagList->tags[i].tagGroup == item.tagGroup) )
    {
      tagList->tags[i] = item;  
      return 0;
    }
  
  // tag was not found - add
  {
    TDivTagItem *newTags = (TDivTagItem *) dimMalloc( fmtHndl, (tagList->count+1) * sizeof(TDivTagItem) );
    if (newTags == NULL) return 1;

    for (i=0; i<tagList->count; i++) newTags[i] = tagList->tags[i];
    newTags[tagList->count] = item;
    dimFree( fmtHndl, tagList->tags );
    tagList->tags = newTags;
    tagList->count++;
  }

  return 0;
}


//------------------------------------------------------------------------------
// PALETTE
//------------------------------------------------------------------------------

HPALETTE createGreyPalette( )
{
  HPALETTE hpal;
  LOGPALETTE *pal;
  int i;
  int palSize; //Size of the palette buffer

  palSize = sizeof(LOGPALETTE) + (sizeof(PALETTEENTRY) * 255);
  pal = (LOGPALETTE *) malloc(palSize);

  pal->palVersion = 0x300;
  pal->palNumEntries = 256;

  // Set the colors to their Gray indexes
  for (i = 0 ; i <= 255; i++)
  {
    pal->palPalEntry[i].peRed   = (unsigned char) i;
    pal->palPalEntry[i].peGreen = (unsigned char) i;
    pal->palPalEntry[i].peBlue  = (unsigned char) i;
    pal->palPalEntry[i].peFlags = PC_NOCOLLAPSE;
  }

  hpal = CreatePalette(pal);
  free(pal);
  return hpal;
}

HPALETTE createHPalette( int num_colors, RGBQUAD *bmiColors )
{
  HPALETTE hpal;
  LOGPALETTE *pal;
  int i;
  int palSize; //Size of the palette buffer

  palSize = sizeof(LOGPALETTE) + (sizeof(PALETTEENTRY) * (num_colors-1) );
  pal = (LOGPALETTE *) malloc(palSize);

  pal->palVersion = 0x300;
  pal->palNumEntries = num_colors;

  // Set the colors to their Gray indexes
  for (i = 0 ; i < num_colors; i++)
  {
    pal->palPalEntry[i].peRed   = (unsigned char) bmiColors[i].rgbRed;
    pal->palPalEntry[i].peGreen = (unsigned char) bmiColors[i].rgbGreen;
    pal->palPalEntry[i].peBlue  = (unsigned char) bmiColors[i].rgbBlue;
    pal->palPalEntry[i].peFlags = PC_NOCOLLAPSE;
  }

  hpal = CreatePalette(pal);
  free(pal);
  return hpal;
}

//------------------------------------------------------------------------------
// image
//------------------------------------------------------------------------------

DIM_UINT getHBLineSizeInBytes( DIM_UINT w, DIM_UINT depth )
{
  return (((w * depth +31) & ~31)/8);
}

DIM_UCHAR *getHBScanLine( DIM_UINT y, DIM_UCHAR *p, DIM_UINT w, DIM_UINT depth )
{
  DIM_UINT ls = getHBLineSizeInBytes( w, depth );
  return p + (ls * y);
}


int dimCreateHBitmap( TDivFormatHandle *fmtHndl, DIM_UINT w, DIM_UINT h, DIM_UINT depth, int num_colors, RGBQUAD *lut, DIM_UCHAR **p )
{
  if (fmtHndl == NULL) return 1;

  int bpp = depth;


  // Allocate memory for the BITMAPINFO structure. (This structure 
  // contains a BITMAPINFOHEADER structure and an array of RGBQUAD 
  // data structures.) 

  BITMAPINFO *pbmi;
  DIM_UINT pbmi_size = 0;

  if (bpp != 24) 
    pbmi_size = sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * (1<< bpp); 
  else 
    // There is no RGBQUAD array for the 24-bit-per-pixel format. 
    pbmi_size = sizeof(BITMAPINFOHEADER);

  pbmi = (PBITMAPINFO) LocalAlloc(LPTR, pbmi_size ); 
  memset(pbmi, 0, pbmi_size );


  // Initialize the fields in the BITMAPINFO structure. 
  pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
  pbmi->bmiHeader.biWidth = w; 
  pbmi->bmiHeader.biHeight = h; 
  pbmi->bmiHeader.biPlanes = 1; 
  pbmi->bmiHeader.biBitCount = bpp; 
  if (bpp < 24) pbmi->bmiHeader.biClrUsed = (1<<bpp); 

  // If the bitmap is not compressed, set the BI_RGB flag. 
  pbmi->bmiHeader.biCompression = BI_RGB; 

  // Compute the number of bytes in the array of color 
  // indices and store the result in biSizeImage. 
  // For Windows NT, the width must be DWORD aligned unless 
  // the bitmap is RLE compressed. This example shows this. 
  // For Windows 95/98/Me, the width must be WORD aligned unless the 
  // bitmap is RLE compressed.
  pbmi->bmiHeader.biSizeImage = ((w * bpp +31) & ~31) /8 * h; 
  // Set biClrImportant to 0, indicating that all of the 
  // device colors are important. 
  pbmi->bmiHeader.biClrImportant = 0; 

  if ( (num_colors > 0) && (lut != NULL) )
  {
    int i;
    for (i=0; i<num_colors; i++)
    {
      pbmi->bmiColors[i].rgbRed   = lut[i].rgbRed;
      pbmi->bmiColors[i].rgbGreen = lut[i].rgbGreen;
      pbmi->bmiColors[i].rgbBlue  = lut[i].rgbBlue;
    }
    fmtHndl->hpal = createHPalette( 256, pbmi->bmiColors );
  }

  fmtHndl->hbmp = CreateDIBSection(NULL, pbmi, DIB_RGB_COLORS, (void**)p, NULL, 0);
  GdiFlush();
  
  if (p == NULL) return 1;

  GdiFlush();
  return 0;
}








