/*******************************************************************************

  PointList is needed to specify shape (Chain Code), e.g. for filters

  Programmer: Dima V. Fedorov <http://www.dimin.net/>

  History:
    03/21/2001 17:17:01 - First Creation

*******************************************************************************/
#include "PointListUnit.h"

#include "limits.h"
#if defined(__BORLANDC__)

//******************************************************************************
// Miscellaneous functions - not necessary to modify
//******************************************************************************

int __fastcall CompareX (void *Item1, void *Item2)
{
  TPoint *Point1, *Point2;
  int Result;

  Result = 0;
  Point1 = (TPoint *) Item1;
  Point2 = (TPoint *) Item2;

  if (Point1->x < Point2->x) Result = -1;
  if (Point1->x == Point2->x)
  {
    Result = 0;
    if (Point1->y < Point2->y) Result = -1;
    if (Point1->y > Point2->y) Result = 1;
  }
  if (Point1->x > Point2->x) Result = 1;

  return Result;
}

int __fastcall CompareY (void *Item1, void *Item2)
{
  TPoint *Point1, *Point2;
  int Result;

  Result = 0;
  Point1 = (TPoint *) Item1;
  Point2 = (TPoint *) Item2;
  
  if (Point1->y < Point2->y) Result = -1;
  if (Point1->y == Point2->y)
  {
    Result = 0;
    if (Point1->x < Point2->x) Result = -1;
    if (Point1->x > Point2->x) Result = 1;
  }
  if (Point1->y > Point2->y) Result = 1;

  return Result;
}


//******************************************************************************
// TPointList
//******************************************************************************
__fastcall TPointList::~TPointList(void)
{
  Clear();
}

int __fastcall TPointList::AddPoint(TPoint Point)
{
  TPoint *NewPPoint;
  int Result;

  NewPPoint = (TPoint *) malloc(sizeof(TPoint));
  *NewPPoint = Point;
  Result = Add(NewPPoint);
  return Result;
}

void __fastcall TPointList::PutPoint(int Index, TPoint Item)
{
  TPoint *NewPPoint;

  // first remove old
  NewPPoint = (TPoint *) Items[Index];
  if (NewPPoint != NULL) free(NewPPoint);

  // now include new
  NewPPoint = (TPoint *) malloc(sizeof(TPoint));
  *NewPPoint = Item;
  Items[Index] = NewPPoint;
}

TPoint __fastcall TPointList::GetPoint(int Index)
{
  TPoint *PPoint;
  PPoint = (TPoint *) Items[Index];
  return *PPoint;
};

void __fastcall TPointList::Clear(void)
{
  TPoint *NewPPoint;
  int i;

  for (i=0; i<=Count-1; i++)
  {
    NewPPoint = (TPoint *) Items[i];
    if (NewPPoint != NULL) free(NewPPoint);
  }
  Clear();
};

void __fastcall TPointList::SortByX(void)
{
  Sort(CompareX);
};

void __fastcall TPointList::SortByY(void)
{
  Sort(CompareY);
};

void __fastcall TPointList::LineBeginEnd(int y, int& LBegin, int& LEnd)
{
  TPoint *MyPPoint;
  int i;

  LBegin = -1;
  LEnd = -2;

  // Find x1
  i = 0;
  while (i < Count)
  {
    MyPPoint = (TPoint *)(Items[i]);
    if (MyPPoint->y == y)
    {
      LBegin = MyPPoint->x;
      break;
    }
    i = i + 1;
  }

  while ( (i < Count) && (MyPPoint->y == y) )
  {
    MyPPoint = (TPoint *) Items[i];
    if (MyPPoint->y == y) LEnd = MyPPoint->x;
    i = i + 1;
  }
}

TRect __fastcall TPointList::EnvolvingRect()
{
  int Top, Bottom, Left, Right;
  int i;

  Top = INT_MAX;
  Bottom = 0;
  Left = INT_MAX;
  Right = 0;

  for (i=0; i<=Count-1; i++)
  {

    if (Top > Points[i].y) Top = Points[i].y;
    if (Bottom < Points[i].y) Bottom = Points[i].y;
    if (Left > Points[i].x) Left = Points[i].x;
    if (Right < Points[i].x) Right = Points[i].x;
  };

  return Rect(Left, Top, Right, Bottom);
};

void __fastcall TPointList::Assign(TPointList *NewList)
{
  int i;
  Clear();
  for (i=0; i<=NewList->Count-1; i++) AddPoint(NewList->Points[i]);
};



void __fastcall TPointList::Delete(int Index)
{
  TPoint *NewPPoint;
  NewPPoint = (TPoint*) this->Items[Index];
  if (NewPPoint != NULL) free(NewPPoint);
}

void __fastcall TPointList::InsertPoint(int Index, TPoint Point)
{
  TPoint *NewPPoint;
  NewPPoint = new TPoint;
  NewPPoint->x = Point.x;
  NewPPoint->y = Point.y;
  this->Insert( Index, NewPPoint);
}

unsigned long __fastcall TPointList::PixelsInside()
{
  long maxy, miny;
  TRect r;
  long i;
  int LBegin, LEnd;

  r = EnvolvingRect();
  maxy = r.Bottom;
  miny = r.Top;
  unsigned long Result = 0;
  for (i=miny; i<maxy; ++i)
  {
    LineBeginEnd(i, LBegin, LEnd);
    if ( (LBegin>0) && (LEnd>0) && (LEnd-LBegin>0) )
      Result = Result + (LEnd-LBegin);
  }

  return Result;
}

TPoint __fastcall TPointList::FirstPoint()
{
  return *((TPoint*) First() );
}

TPoint __fastcall TPointList::LastPoint()
{
  return *((TPoint*) Last() );
}

void __fastcall TPointList::RemoveRepeatedNoSort()
{
  int i;
  if (Count==1) return;
  for (i=Count-1; i>1; --i)
  {
    if ( ( Points[i].x == Points[i-1].x ) && ( Points[i].y == Points[i-1].y ) )
      Delete( i );
  }
}

void __fastcall TPointList::RemoveRepeated()
{
  int i;

  if (Count==1) return;
  SortByY();

  for (i=Count-1; i>1; --i)
  {
    if ( ( Points[i].x == Points[i-1].x ) && ( Points[i].y == Points[i-1].y ) )
      Delete( i );
  }
}

#endif


