/*******************************************************************************

  Example Plugin Library (DLL) for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    12/05/2000 20:36:48 - First Creation
    03/25/2001 17:11:55 - Conversion to v2.1
    06/20/2004 01:10:00 - DIV plugin interface v5

*******************************************************************************/

#include <vcl.h>
#pragma hdrstop

#include <windows.h>

#include "div5_filters_plugin.h"

TDIVPluginHeader PluginHeader = {
  sizeof( TDIVPluginHeader ),
  5, // plugin interface version - MUST BE 5
  DIV_PLUGIN_TYPE_FILTER,
  1, // plugin version
  6, // four filters
  "DIMIN Filter Example (CBUILDER)"
};

//------------------------------------------------------------------------------
// About procedure
//------------------------------------------------------------------------------
void __stdcall aboutPlugin( HWND hwndParent, HINSTANCE hDllInstance ) 
{
  MessageBox(GetFocus(), "DIMIN Filter Example (MFC)", "About Plug-in", MB_OK | MB_ICONEXCLAMATION);
}

//------------------------------------------------------------------------------
// Filter 1 shows simple no parameter filter
//------------------------------------------------------------------------------

TDIVFilterHeader FilterHeader1 = {
  sizeof(TDIVFilterHeader), // size of currently used struct
  "EXMPL_RAINT_RED",
  "24bpp Image - Paint in red",
  filter1, // pointer to a TPluginFilter function
  0,       // Bpp needed by filter, 0 - any, 888 - Gray, 111-mono
  NULL,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin     // TDivAboutProc   aboutProc    // about proc provided by filter
};


//------------------------------------------------------------------------------
// Filter 2 defines one parameter to be set by DIV
//------------------------------------------------------------------------------

TDIVTagedArgument filter2ArgsArr = {
  1000,           // id number, plugin may use ids above 1000 as private
  DIM_TAG_LONG,   // TDimTagType
  1,              // number of records of type tagType pointed by tagData
  "Color Value:",  // text description of the argument
  (void *) 255,   // pointer to data, if data size is <= 4 bytes
                  // then it's directly stored into pointer
  (void *) 0,     // pointer to min val, used if needed to define
                  // a special value retrieval, e.g. -100 .. 100
  (void *) 255    // pointer to max val
};


TDIVArguments filter2Args = {
  1,          // 0 - no preview window, 1 - DIV will open preview
  1,          // number of arguments in the array
  &filter2ArgsArr, // pointer to array of argumants
  NULL, // custom controls creation INIT
  NULL, // custom controls creation QUIT
};

TDIVFilterHeader FilterHeader2 = {
  sizeof(TDIVFilterHeader),
  "EXMPL_RAINT_COLOR",
  "Gray Image - Paint in Color",
  filter2, // filter
  888,     // 8bpp needed
  &filter2Args, // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

//------------------------------------------------------------------------------
// Filter 3 shows how to create a new image using GDI
//------------------------------------------------------------------------------

TDIVFilterHeader FilterHeader3 = {
  sizeof(TDIVFilterHeader),
  "EXMPL_NEW_PAL_IMG",
  "8bpp - New paletted image",
  filter3, // filter
  0,       // 8bpp needed
  NULL,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

//------------------------------------------------------------------------------
// Filter 4 shows how to create custom controls in the window provided by DIV
//------------------------------------------------------------------------------

TDIVArguments filter4Args = {
  1,          // 0 - no preview window, 1 - DIV will open preview
  1,          // number of arguments in the array
  NULL, // don't really need anything here, we set our arguments ourselves
        // using createArgControls4
  createArgControls4, // custom controls creation INIT
  freeArgControls4, // custom controls creation QUIT
};

TDIVFilterHeader FilterHeader4 = {
  sizeof(TDIVFilterHeader),
  "EXMPL_CREATE_CNTRL",
  "Create custom controls",
  filter4, // filter
  0,     
  &filter4Args,
  aboutPlugin
};


//------------------------------------------------------------------------------
// Filter 5 rotates provided image by 90 degrees
//------------------------------------------------------------------------------

TDIVFilterHeader FilterHeader5 = {
  sizeof(TDIVFilterHeader),
  "EXMPL_ROT_90_DEG",
  "Rotate image by 90 degrees",
  filter5, // filter
  0,       // no bpp needed
  NULL,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

//------------------------------------------------------------------------------
// Filter 6 shows simple usage of VCL TBitmap
//------------------------------------------------------------------------------

TDIVFilterHeader FilterHeader6 = {
  sizeof(TDIVFilterHeader), // size of currently used struct
  "EXMPL_VCL_TBITMAP",
  "VCL TBitmap Example",
  filterVCL, // pointer to a TPluginFilter function
  0,       // Bpp needed by filter, 0 - any, 888 - Gray, 111-mono
  NULL,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin     // TDivAboutProc   aboutProc    // about proc provided by filter
};


//---------------------------------------------------------------------------

int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void*)
{
        return 1;
}

extern "C" __declspec(dllexport)
TDIVPluginHeader* __stdcall divGetPluginHeader() // WINAPI
{
   return &PluginHeader;
}

extern "C" __declspec(dllexport)
TDIVFilterHeader* __stdcall divGetFilterHeader(int I)
{
  switch (I)
  {
    case 0: return &FilterHeader1;
    case 1: return &FilterHeader2;
    case 2: return &FilterHeader3;
    case 3: return &FilterHeader4;
    case 4: return &FilterHeader5;
    case 5: return &FilterHeader6;
    default : return NULL;
  }
}

extern "C" __declspec(dllexport)
TDivPluginFilter __stdcall divGetFilter(int I)
{
  switch (I)
  {
    case 0: return filter1;
    case 1: return filter2;
    case 2: return filter3;
    case 3: return filter4;
    case 4: return filter5;
    case 5: return filterVCL;
    default: return NULL;
  }
}

//---------------------------------------------------------------------------
